from .guilds_manager import GuildsManager

class DatabaseManager:

  """
  Manages database
    
  Path: db

  Attributes:
    guild (GuildsManager): manages guild docs
  """

  def __init__(self):

    self.guilds = GuildsManager()


from dpy_firestore_manager.structures import Collection
from .guild_manager import GuildManager

class GuildsManager(Collection):

  """
  Manages the guild docs of the guilds collection
    
  Path: db/guilds
  """
  
  def __init__(self):

    super().__init__('guilds')


  async def get(self, id = None):

    """
    Gets field values of doc, gets all docs if no id is passed

    Args:
      id (str, optional): id of doc to get

    Returns:
      dict: field values of doc
    """

    doc = await self.data(id)
    return doc


  async def post(self, data):

    """
    Posts field values of doc

    Args:
      data (dict): field values to update
      data.id (str): id of doc to update

    Returns:
      dict: updated field values of doc
    """

    docRef = await self.reference(data.get("id"))
    docRef.set(data, merge=True)
    return data


  async def delete(self, id):

    """
    Deletes doc

    Args:
      id (str): id of doc to delete

    Returns:
      dict: field values of deleted doc
    """

    docRef = await self.reference(id)
    doc = await self.data(id)
    docRef.delete()
    return doc


  async def fetch(self, id):

    """
    Fetches doc, fetches a pseudo doc if doc does not exist

    Args:
      id (str): id of doc to fetch

    Returns:
      GuildManager: manages fetched guild
    """

    doc = await self.data(id)
    if (doc is None):
      return GuildManager({"id": id})
    else:
      return GuildManager(doc)


from .database import Database

class Collection(Database):

  """
  Collection structure

  Args:
    path (str): absolute path leading to the collection

  Attributes:
    path (str): absolute path leading to the collection
  """
  
  def __init__(self, path):

    super().__init__()
    self.path = path


  async def reference(self, id = None):

    """
    Gets doc reference, gets collection reference if no id is passed

    Args:
      id (str, optional): id of doc to get

    Returns:
      Firebase.docRef | Firebase.collectionRef: reference to doc
    """

    if (id is None):
      collectionRef = self.db.collection(f'{self.path}')
      return collectionRef
    else:
      docRef = self.db.document(f'{self.path}/{id}')
      return docRef


  async def data(self, id = None):

    """
    Gets doc field values, gets all doc from collection if no id is passed

    Args:
      id (str, optional): id of doc to get

    Returns:
      dict | Arr<dict>: field values doc
    """

    if (id is None):
      collectionRef = await self.reference()
      docs = collectionRef.stream()
      collection = []
      for doc in docs:
        data = doc.to_dict()
        collection.append(data)
      return collection
    else:
      docRef = await self.reference(id)
      _doc = docRef.get()
      doc = _doc.to_dict()
      return doc


from dpy_firestore_manager.structures import Document
from dpy_firestore_manager.utilities import assign
from .members_manager import MembersManager

class GuildManager(Document):

  """
  Manages a specific guild doc of the guilds collection
    
  Path: db/guilds/{guild_id}

  Args:
    data (dict): field values of doc
    data.id (str): id of guild doc 
  """

  def __init__(self, data):

    super().__init__(f'guilds/{data["id"]}')
    assign(self, data)

    self.members = MembersManager(self.id)


  async def create(self, guild):

    """
    Creates doc containing basic information

    Args:
      guild (discord.Guild): guild to create
    """

    await self.update({"id": guild.id, "name": guild.name})


  async def update(self, data):

    """
    Updates doc

    Args:
      data (dict): field values to update
    """

    docRef = await self.reference()
    docRef.set(data, merge=True)


from dpy_firestore_manager.structures import Document
from dpy_firestore_manager.utilities import assign

class MemberManager(Document):

  """
  Manages a specific member doc of the members collection
    
  Path: db/guilds/{guild_id}/members/{member_id}

  Args:
    data (dict): field values of doc
    data.id (str): id of member doc 
  """

  def __init__(self, guild_id, data):

    super().__init__(f'guilds/{guild_id}/members/{data["id"]}')
    assign(self, data)

    self.guild_id = guild_id
      

  async def create(self, member):

    """
    Creates doc containing basic information

    Args:
      member (discord.Member): member to create
    """

    await self.update({"id": member.id, "name": member.name})


  async def update(self, data):

    """
    Updates doc

    Args:
      data (dictionary): field values to update
    """

    docRef = await self.reference()
    docRef.set(data, merge=True)


from dpy_firestore_manager.structures import Collection
from .member_manager import MemberManager

class MembersManager(Collection):

  """
  Manages the member docs of a specific guild members collection
    
  Path: db/guilds/{guild_id}/members

  Args:
    guild_id (str): guild id of members collection
  """

  def __init__(self, guild_id):
    
    super().__init__(f'guilds/{guild_id}/members')
    self.guild_id = guild_id


  async def get(self, id = None):

    """
    Gets field values of doc, gets all docs if no id is passed

    Args:
      id (str, optional): id of doc to get

    Returns:
      dict: field values of doc
    """

    doc = await self.data(id)
    return doc


  async def post(self, data):

    """
    Posts field values of doc

    Args:
      data (dict): field values to update
      data.id (str): id of doc to update

    Returns:
      dict: updated field values of doc
    """

    docRef = await self.reference(data.get("id"))
    docRef.set(data, merge=True)
    return data


  async def delete(self, id):

    """
    Deletes doc

    Args:
      id (str): id of doc to delete

    Returns:
      dict: field values of deleted doc
    """

    docRef = await self.reference(id)
    doc = await self.data(id)
    docRef.delete()
    return doc


  async def fetch(self, id):

    """
    Fetches doc, fetches a pseudo doc if doc does not exist

    Args:
      id (str): id of doc to fetch

    Returns:
      MemberManager: manages fetched member
    """

    doc = await self.data(id)
    if (doc is None):
      return MemberManager(self.guild_id, {"id": id})
    else:
      return MemberManager(self.guild_id, doc)
  


from firebase_admin import firestore

class Database:

  """
  Database structure

  Attributes:
    db (Firestore.client): access Firestore
  """

  def __init__(self):

    self.db = firestore.client()


import os
from dotenv import load_dotenv
import firebase_admin
from firebase_admin import credentials
from .managers import DatabaseManager

class Client:

  """
  Main class for interacting with your Firestore

  Attributes:
    db (DatabaseManager): manages database
  """

  def __init__(self):

    load_dotenv()
    cred = credentials.Certificate({
      "type": "service_account",
      "project_id": os.getenv('PROJECT_ID'),
      "private_key_id": os.getenv('PRIVATE_KEY_ID'),
      "private_key": os.getenv('PRIVATE_KEY'),
      "client_email": os.getenv('CLIENT_EMAIL'),
      "client_id": os.getenv('CLIENT_ID'),
      "auth_uri": "https://accounts.google.com/o/oauth2/auth",
      "token_uri": "https://oauth2.googleapis.com/token",
      "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
      "client_x509_cert_url": os.getenv('CLIENT_X509_CERT_URL')
    })
    firebase_admin.initialize_app(cred)

    self.db = DatabaseManager()


  async def get_prefix(self, bot, message):

    """
    Gets guild prefix

    Args:
      bot (discord.Client): client
      message (discord.Message): message

    Returns:
      str: guild-unique prefix 
    """

    id = message.guild.id
    guild = await self.db.guilds.fetch(id)
    self.guild = guild
    self.prefix = guild.prefix
    return self.prefix


from setuptools import setup, find_packages

with open('README.md', 'r', encoding = 'utf-8') as f:
  long_description = f.read()

setup(
  name='dpy-firestore-manager',
  version='0.0.1',
  description='Hello',
  long_description=long_description,
  long_description_content_type="text/markdown",
  packages=find_packages(),
  install_requires = [
    "discord.py>=1.0.1",
    "firebase_admin>=5.0.0"
  ]
)


from .database import Database

class Document(Database):

  """
  Document structure

  Args:
    path (str): absolute path leading to the document

  Attributes:
    path (str): absolute path leading to the document
  """
  
  def __init__(self, path):

    super().__init__()
    self.path = path


  async def reference(self):

    """
    Gets doc reference

    Returns:
      Firebase.docRef: reference to doc
    """
    
    return self.db.doc(f'{self.path}')


  async def data(self):

    """
    Gets doc field values

    Returns:
      dict: field values doc
    """

    docRef = await self.reference()
    doc = docRef.get()
    return doc.to_dict()